package com.netifera.platform.internal.channel.tcplisten;

import java.net.InetAddress;
import java.net.InetSocketAddress;

import com.netifera.platform.channel.tcplisten.ITCPListenServer;

public class ConfigParser {
	public final static int INVALID_PORT = -1;

	
	/*
	 * channelConfig is expected in the form "tcplisten:a.b.c.d:port"
	 * for example "tcplisten:22.99.7.12:443"
	 */
	public InetSocketAddress configToAddress(String channelConfig) {
		if(!isValidConfig(channelConfig))
			return null;
		
		final InetAddress address = configToInetAddress(channelConfig);
		final int port = configToPort(channelConfig);
		return new InetSocketAddress(address, port);
	}
	
	private boolean isValidConfig(String config) {
		
		try {
			final String[] parts = splitConfig(config);
		
			return parts[0].equals(ITCPListenServer.CHANNEL_TYPE) &&
				IPv4Address.isValid(parts[1]) &&
				stringToPort(parts[2]) != INVALID_PORT;
		} catch(IllegalArgumentException e) {
			return false;
		}
	}
	
	private InetAddress configToInetAddress(String config) {
		final String[] parts = splitConfig(config);
		final int addressData = IPv4Address.stringParse(parts[1]);
		final IPv4Address address = new IPv4Address(addressData);
		return address.getInetAddress();
	}
	
	private int configToPort(String config) {
		final String[] parts = splitConfig(config);
		return stringToPort(parts[2]);
	}
	
	private String[] splitConfig(String config) {
		final String[] parts = config.split(":");
		if(parts.length != 3) {
			throw new IllegalArgumentException();
		} else {
			return parts;
		}
	}
	public int stringToPort(String s) {
		try {
			final int port = Integer.parseInt(s);
			return (isValidPort(port)) ? (port) : (INVALID_PORT);
		} catch(NumberFormatException e) {
			return INVALID_PORT;
		}	
	
	}
	
	private boolean isValidPort(int port) {
		return port > 0 && port <= 65535;
	}

}
