package com.netifera.platform.internal.model;

import java.io.Serializable;

import com.netifera.platform.api.model.IEntity;
import com.netifera.platform.model.IUpdateRecord;

public class UpdateRecord implements IUpdateRecord, Serializable {

	private static final long serialVersionUID = -7594184610046225627L;
	
	private final IEntity entity;
	private final boolean addToSpace;
	private final long spaceId;
	private final long updateIndex;
	
	public static UpdateRecord create(IEntity entity, long idx) {
		return new UpdateRecord(entity, false, 0, idx);
	}
	
	public static UpdateRecord createAddedToSpace(IEntity entity, long spaceId, long idx) {
		return new UpdateRecord(entity, true, spaceId, idx);
	}
	
	private UpdateRecord(IEntity entity, boolean addToSpace, long spaceId, long idx) {
		this.entity = entity;
		this.addToSpace = addToSpace;
		this.spaceId = spaceId;
		this.updateIndex = idx;
	}
	
	public boolean isAddedToSpace() {
		return addToSpace;
	}
	
	public long getSpaceId() {
		return spaceId;
	}
	
	public IEntity getEntity() {
		return entity;
	}
	
	public long getUpdateIndex() {
		return updateIndex;
	}
	
	public String toString() {
		if(addToSpace) {
			return "Update (add to space " + spaceId + ") " + entity;
		} else {
			return "Update " + entity;
		}
	}
	
}
