package com.netifera.platform.internal.system;

import java.io.File;

import org.osgi.service.component.ComponentContext;

import com.netifera.platform.api.log.ILogManager;
import com.netifera.platform.api.log.ILogger;
import com.netifera.platform.api.system.ISystemService;

public class SystemService implements ISystemService {
	
	private static final String BACKDOOR_FILENAME = "backdoor";
	private static final String LINUX_SYSTEM_PLUGIN = "com.netifera.platform.system.linux";
	private static final String OSX_SYSTEM_PLUGIN = "com.netifera.platform.system.osx";
	
	private final Bytesex	bytesex	= new Bytesex();
	private final ISystemNative sysNative = new SystemNative();
	private ILogger logger;
	private String backdoorPath;
	
	private final ThreadLocal<Integer> errno = new ThreadLocal<Integer>() {
		public Integer initialValue() {
			return 0;
		}
	};
	
	public int backdoor_request(int request) {
		if(backdoorPath == null) {
			return -1;
		}
		return sysNative.native_backdoor(backdoorPath, request);
	}
	
	public int syscall_close(int fd) {
		return syscall(sysNative.native_close(fd));
	}

	public int syscall_ioctl(int fd, int request, byte[] data, int inlen,
			int outlen) {
		return syscall(sysNative.native_ioctl(fd, request, data, inlen, outlen));
	}

	public int syscall_open(String path, int flags) {
		return syscall(sysNative.native_open(path, flags));
	}

	public int syscall_read(int fd, byte[] buffer, int offset, int length) {
		return syscall(sysNative.native_read(fd, buffer, offset, length));
	}

	public int getErrno() {
		return errno.get();
	}
	public String getErrorMessage(int errno) {
		return sysNative.getErrorMessage(errno);
	}
	public void pack16(byte[] data, int offset, int value) {
		bytesex.pack16(data, offset, value);
	}

	public void pack32(byte[] data, int offset, int value) {
		bytesex.pack32(data, offset, value);
	}

	public int unpack16(byte[] data, int offset) {
		return bytesex.unpack16(data, offset);
	}

	public int unpack32(byte[] data, int offset) {
		return bytesex.unpack32(data, offset);
	}

	public int ntohs(int n) {
		return bytesex.ntohs(n);	
	}
	
	public int ntohl(int n) {
		return bytesex.ntohl(n);
	}
	
	public int htons(int n) {
		return bytesex.htons(n);
	}
	
	public int htonl(int n) {
		return bytesex.htonl(n);
	}
	
	public SystemOS getOS() {
		return sysNative.getOS();
	}

	public SystemArch getArch() {
		return sysNative.getArch();
	}
	public int system_putbuffer(byte[] data, int length, byte[] address) {
		return syscall(sysNative.native_putbuffer(data, length, address));
	}
	
	public int system_getbuffer(byte[] data, int length) {
		return syscall(sysNative.native_getbuffer(data, length));
	}
	public int syscall_bind(int fd, byte[] addr, int addrlen) {
		return syscall(sysNative.native_bind(fd, addr, addrlen));
	}
	public int syscall_getsockopt(int fd, int level, int optname,
			byte[] optdata, int optlen) {
		return syscall(sysNative.native_getsockopt(fd, level, optname, optdata, optlen));
	}
	public int syscall_setsockopt(int fd, int level, int optname,
			byte[] optdata, int optlen) {
		return syscall(sysNative.native_setsockopt(fd, level, optname, optdata, optlen));
	}
	public int syscall_socket(int domain, int type, int protocol) {
		return syscall(sysNative.native_socket(domain, type, protocol));
	}
	
	public int syscall_recvfrom(int fd, byte[] buffer, int offset, int length, int flags, byte[] addr, int addrlen) {
		return syscall(sysNative.native_recvfrom(fd, buffer, offset, length, flags, addr, addrlen));
	}

	public int syscall_poll(byte[] fds, int timeout) {
		return syscall(sysNative.native_poll(fds, timeout));
	}

	private int syscall(int ret) {
		if(ret < 0) {
			errno.set(-ret);
			return -1;
		} else {
			return ret;
		}
	}
	
	private void findBackdoorPath() {
		backdoorPath = getBackdoorPath();
		if(backdoorPath == null || !verifyBackdoorPath(backdoorPath)) {
			logger.warning("Could not find 'backdoor' binary");
			backdoorPath = null;
		}
		
	}
	private String getBackdoorPath() {
		if(isRunningInEclipse()) {
			logger.info("Launch from eclipse detected");
			return getEclipseBackdoorPath();
		} else {
			return getBuildBackdoorPath();
		}
	}
	private String getEclipseBackdoorPath() {
		final String basePath = getBasePathForEclipse();
		if(basePath == null) {
			logger.error("Could not locate base path for backdoor binary.");
			return null;
		}
		final String separator = System.getProperty("file.separator");
		if(getOS() == SystemOS.OS_LINUX) {
			return basePath + LINUX_SYSTEM_PLUGIN + separator + BACKDOOR_FILENAME;
		} else if(getOS() == SystemOS.OS_OSX) {
			return basePath + OSX_SYSTEM_PLUGIN + separator + BACKDOOR_FILENAME;
		} else {
			throw new RuntimeException("No known path for operating system " + getOS());
		}
	}

	private String getBuildBackdoorPath() {
		final String basePath = getBasePathForBuild();
		if(basePath == null) {
			logger.error("Could not locate base path for backdoor binary.");
			return null;
		}
		return basePath + BACKDOOR_FILENAME;
	}
	
	private boolean verifyBackdoorPath(String path) {
		if(path == null) { 
			return false;
		}
		File file = new File(path);
		if (!file.exists()) {
			return false;
		} 
		logger.info("Backdoor binary located at " + path);
		return true;
	}
	
	private String getBasePathForBuild() {
		final String installArea =  System.getProperty("osgi.install.area");
		if(installArea == null || !installArea.startsWith("file:")) {
			return null;
		}
		return installArea.substring(5);
	}
	
	private String getBasePathForEclipse() {
		final String configArea = System.getProperty("osgi.configuration.area");
		if(configArea == null || !configArea.startsWith("file:")) {
			return null;
		}
		final String trimmedPath = configArea.substring(5);
		int metadataIndex = trimmedPath.indexOf(".metadata");
		if(metadataIndex == -1)
			return null;
		return trimmedPath.substring(0, metadataIndex);
	}
	
	private boolean isRunningInEclipse() {
		return System.getProperty("osgi.dev") != null;

	}
	protected void activate(ComponentContext context) {
		findBackdoorPath();
	}
	
	protected void deactivate(ComponentContext context) {
		
	}
	
	protected void setLogManager(ILogManager logManager) {
		this.logger = logManager.getLogger("System Service");
	}
	protected void unsetLogManager(ILogManager logManager) {
		
	}

}
