package com.netifera.platform.ui.api.model;

import org.eclipse.swt.graphics.Image;

import com.netifera.platform.api.model.IShadowEntity;
/**
 * This interface implements an OSGi service which extends the Entity Label Provider
 * Service.
 * 
 * @see com.netifera.platform.ui.api.model.IEntityLabelProviderService
 *
 */
public interface IEntityLabelProvider {
	
	/**
	 * Returns a <code>String</code> containing the tooltip that should be displayed 
	 * for the given entity.  If a tooltip cannot be generated or if no tooltip should
	 * be displayed, this method must return <code>null</code>.
	 * @param e The entity to provide a tooltip for.
	 * @return Tooltip string.
	 */
	String getToolTipText(IShadowEntity e);
	
	/**
	 * Returns a <code>String</code> which describes the given entity when displayed in 
	 * GUI views.
	 * @param e The entity to provide a description for.
	 * @return String describing the entity.
	 */
	String getText(IShadowEntity e);

	/**
	 * @param e The entity to provide a description for.
	 * @return String describing the entity.
	 */
	String getFullText(IShadowEntity e);

	/**
	 * 
	 * @param e
	 * @return
	 */
	Image getImage(IShadowEntity e);
	
	/**
	 * 
	 * @param e
	 * @return
	 */
	String getInformation(IShadowEntity e);
	
	int getSortingCategory(IShadowEntity e);
	
	/**
	 * 
	 * @param e1
	 * @param e2
	 * @return -2 if not handled
	 */
	int compare(IShadowEntity e1, IShadowEntity e2);
	
	/**
	 * 
	 */
	void dispose();
}
