package com.netifera.platform.ui.probe.status;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import com.netifera.platform.api.events.IEvent;
import com.netifera.platform.api.events.IEventHandler;
import com.netifera.platform.api.probe.IProbe;
import com.netifera.platform.ui.images.ImageCache;
import com.netifera.platform.ui.probe.Activator;

public class ProbeStatusLine extends ContributionItem {

	private CLabel label;
	private IProbe probe;
	private IEventHandler probeChangeListener;
	
	private final static String PROBE_DISCONNECTED = "icons/probe_disconnected.png";
	private final static String PROBE_CONNECTING = "icons/probe_connecting.png";
	private final static String PROBE_CONNECTED = "icons/probe_connected.png";
	private final static String PROBE_FAILED = "icons/probe_failed.png";

	private final ImageCache images = new ImageCache(Activator.PLUGIN_ID);

	public ProbeStatusLine(String id) {
		super(id);
	}
	
	@Override
	public void fill(Composite parent) {
		new Label(parent, SWT.SEPARATOR);
		label = new CLabel(parent, SWT.SHADOW_NONE);
		probeChangeListener = createProbeChangeHandler(parent.getDisplay());
		Activator.getDefault().getProbeManager().addProbeChangeListener(probeChangeListener);
	}

	private IEventHandler createProbeChangeHandler(final Display display) {
		return new IEventHandler() {
			public void handleEvent(IEvent event) {
				display.asyncExec(new Runnable() {
					public void run() {
						update();
					}
				});
			}
		};
	}
	
	@Override
	public void dispose() {
		Activator.getDefault().getProbeManager().removeProbeChangeListener(probeChangeListener);
		super.dispose();
	}

	
	public void update(IProbe probe) {
		this.probe = probe;
		update();
	}
	
	@Override
	public void update() {
		if (label.isDisposed())
			return;
		label.setText(probe.getName());
		switch (probe.getConnectState()) {
		case DISCONNECTED:
			label.setImage(images.get(PROBE_DISCONNECTED));
			label.setToolTipText(null);
			break;
		case CONNECTING:
			label.setImage(images.get(PROBE_CONNECTING));
			label.setToolTipText(null);
			break;
		case CONNECTED:
			label.setImage(images.get(PROBE_CONNECTED));
			label.setToolTipText(null);
			break;
		case CONNECT_FAILED:
			label.setImage(images.get(PROBE_FAILED));
			label.setToolTipText(probe.getConnectError());
		}
	}
}
