package com.netifera.platform.ui.spaces.editors;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistableEditor;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import com.netifera.platform.api.events.IEvent;
import com.netifera.platform.api.events.IEventHandler;
import com.netifera.platform.api.model.ISpace;
import com.netifera.platform.api.model.SpaceNameChangeEvent;
import com.netifera.platform.ui.internal.spaces.Activator;
import com.netifera.platform.ui.spaces.SpaceEditorInput;

public class SpaceEditor extends EditorPart implements IPersistableEditor {
	public final static String ID = "com.netifera.platform.editors.spaces";

	private ISpace space;
	private String visualization = "Tree";
	private ContentViewer viewer;

	public SpaceEditor() {
		super();
	}

	@Override
	public void doSave(IProgressMonitor monitor) {

	}

	@Override
	public void doSaveAs() {

	}

	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		setSite(site);
		setInput(input);
		setPartName(input.getName());
		if(!(input instanceof SpaceEditorInput)) {
			throw new PartInitException("SpaceEditor passed unexpected input type");
		}

		space = ((SpaceEditorInput)input).getSpace();
		space.addChangeListener(new IEventHandler() {
			public void handleEvent(IEvent event) {
				if(event instanceof SpaceNameChangeEvent) {
					setPartName(((SpaceNameChangeEvent)event).getName());
				}
			}
		}
		);

	}
	
	@Override
	public boolean isDirty() {
		return false;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void createPartControl(Composite parent) {
		viewer = Activator.getDefault().getVisualizationFactory().create(visualization, parent, space);
	}
	
	public void setVisualization(String name) {
		Composite parent = viewer.getControl().getParent();
		viewer.getControl().dispose();
		visualization = name;
		createPartControl(parent);
		parent.layout();
	}

	public void dispose() {
		super.dispose();
		space.close();
	}

	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public void restoreState(IMemento memento) {
		visualization = memento.getString("visualization");
		if (visualization == null) visualization = "Tree";
	}

	public void saveState(IMemento memento) {
		memento.putString("visualization", visualization);
	}
}
